@@ -5,6 +5,7 @@ import ctypes
 import json
 import urllib
 import logging
+import uuid
 import webbrowser
 from collections.abc import Callable
 from threading import Semaphore, Thread, main_thread
@@ -12,14 +13,16 @@ from threading import Semaphore, Thread, main_thread
 import AppKit
 import Foundation
 import WebKit
-from objc import _objc, nil, selector, super
+from objc import _objc, nil, selector, super, lookUpClass, classAddMethod
+
 from PyObjCTools import AppHelper
 
-from webview import (FOLDER_DIALOG, OPEN_DIALOG, SAVE_DIALOG, _state, parse_file_type, windows, settings as webview_settings)
+from webview import (FileDialog, _state, windows, settings as webview_settings)
 from webview.dom import _dnd_state
 from webview.menu import Menu, MenuAction, MenuSeparator
+from webview.models import Request, Response
 from webview.screen import Screen
-from webview.util import DEFAULT_HTML, create_cookie, js_bridge_call, inject_pywebview
+from webview.util import DEFAULT_HTML, create_cookie, js_bridge_call, inject_pywebview, parse_file_type, stringify_headers
 from webview.window import FixPoint
 
 
@@ -49,11 +52,12 @@ logger.debug('Using Cocoa')
 renderer = 'wkwebview'
 
 
+
 class BrowserView:
     instances = {}
     app = AppKit.NSApplication.sharedApplication()
     app.setActivationPolicy_(0)
-    app_menu_list = None
+    current_menu = None
 
     cascade_loc = Foundation.NSMakePoint(100.0, 0.0)
 
@@ -72,6 +76,15 @@ class BrowserView:
             return self.focus
 
     class WindowDelegate(AppKit.NSObject):
+        def windowDidBecomeKey_(self, notification):
+            i = BrowserView.get_instance('window', notification.object())
+
+            if i.menu and BrowserView.current_menu != i.menu:
+                BrowserView.current_menu = i.menu
+                new_menu = i._recreate_menus(BrowserView.current_menu)
+                BrowserView.app.setMainMenu_(new_menu)
+
+
         def windowShouldClose_(self, window):
             i = BrowserView.get_instance('window', window)
             return BrowserView.should_close(i.pywebview_window)
@@ -207,13 +220,26 @@ class BrowserView:
             result = BrowserView.display_confirmation_dialog(ok, cancel, message)
             handler(result)
 
+        # Display a Javascript input panel
+        def webView_runJavaScriptTextInputPanelWithPrompt_defaultText_initiatedByFrame_completionHandler_(
+            self, webview, prompt, default_text, frame, handler
+        ):
+            i = BrowserView.get_instance('webview', webview)
+            ok = i.localization['global.ok']
+            cancel = i.localization['global.cancel']
+
+            result = BrowserView.display_input_dialog(ok, cancel, prompt, default_text)
+            handler(result)
+
         # Display an open panel for <input type="file"> element
         def webView_runOpenPanelWithParameters_initiatedByFrame_completionHandler_(
             self, webview, param, frame, handler
         ):
             i = list(BrowserView.instances.values())[0]
+            file_filter = param._acceptedMIMETypes()
+
             files = i.create_file_dialog(
-                OPEN_DIALOG, '', param.allowsMultipleSelection(), '', [], main_thread=True
+                FileDialog.OPEN, '', param.allowsMultipleSelection(), '', file_filter, main_thread=True
             )
 
             if not handler.__block_signature__:
@@ -261,14 +287,51 @@ class BrowserView:
                     handler(getattr(WebKit, 'WKNavigationActionPolicyCancel', 0))
                     return
 
-            # Normal navigation, allow
-            handler(getattr(WebKit, 'WKNavigationActionPolicyAllow', 1))
+            request = action.request()
+            url = request.URL()
+
+            original_headers = dict(request.allHTTPHeaderFields())
+            i = BrowserView.get_instance('webview', webview)
+
+            if (
+                len(i.pywebview_window.events.request_sent) > 0 and
+                'X-Handled' not in original_headers and
+                str(url) != 'about:blank'
+            ):
+
+                request_ = Request(str(url), request.HTTPMethod(), original_headers)
+                i.pywebview_window.events.request_sent.set(request_)
+                request_.headers['X-Handled'] = 'true'
+
+                new_request = Foundation.NSMutableURLRequest.requestWithURL_(url)
+                new_request.setHTTPMethod_(request.HTTPMethod())
+                new_request.setAllHTTPHeaderFields_(AppKit.NSDictionary(stringify_headers(request_.headers)))
+                new_request.setHTTPBody_(request.HTTPBody())
+                new_request.setHTTPBodyStream_(request.HTTPBodyStream())
+                new_request.setHTTPShouldHandleCookies_(request.HTTPShouldHandleCookies())
+                new_request.setHTTPShouldUsePipelining_(request.HTTPShouldUsePipelining())
+                new_request.setMainDocumentURL_(request.mainDocumentURL())
+                new_request.setNetworkServiceType_(request.networkServiceType())
+                webview.loadRequest_(new_request)
+                handler(getattr(WebKit, 'WKNavigationActionPolicyCancel', 1))
+            else:
+                # Normal navigation, allow
+                handler(getattr(WebKit, 'WKNavigationActionPolicyAllow', 1))
 
         def webView_navigationAction_didBecomeDownload_(self, webview, navigationAction, download):
             download.setDelegate_(BrowserView.DownloadDelegate.alloc().init().retain())
 
         def webView_decidePolicyForNavigationResponse_decisionHandler_(self, webview, navigationResponse, decisionHandler):
             if navigationResponse.canShowMIMEType():
+                response_ = navigationResponse.response()
+                headers = dict(response_.allHeaderFields())
+                response = Response(
+                    response_.URL().absoluteString(),
+                    response_.statusCode(),
+                    headers
+                )
+                webview.pywebview_window.events.response_received.set(response)
+
                 decisionHandler(WebKit.WKNavigationResponsePolicyAllow)
             elif webview_settings['ALLOW_DOWNLOADS']:
                 decisionHandler(WebKit.WKNavigationResponsePolicyCancel)
@@ -321,6 +384,9 @@ class BrowserView:
 
                 inject_pywebview('cocoa', i.js_bridge.window)
 
+                if _state['debug'] and webview_settings['OPEN_DEVTOOLS_IN_DEBUG']:
+                    BrowserView._open_web_inspector(webview)
+
         # Handle JavaScript window.print()
         def userContentController_didReceiveScriptMessage_(self, controller, message):
             if message.body() == 'print':
@@ -462,6 +528,7 @@ class BrowserView:
 
             super(BrowserView.WebKitHost, self).keyDown_(event)
 
+
     def __init__(self, window):
         BrowserView.instances[window.uid] = self
         self.uid = window.uid
@@ -505,6 +572,8 @@ class BrowserView:
                 | AppKit.NSTexturedBackgroundWindowMask
             )
 
+        self.menu = window.menu or _state['menu']
+
         # The allocated resources are retained because we would explicitly delete
         # this instance when its window is closed
         self.window = (
@@ -527,7 +596,8 @@ class BrowserView:
         frame.size.height = window.initial_height
         self.window.setFrame_display_(frame, True)
 
-        self.webview = BrowserView.WebKitHost.alloc().initWithFrame_(rect).retain()
+        config = WebKit.WKWebViewConfiguration.alloc().init()
+        self.webview = BrowserView.WebKitHost.alloc().initWithFrame_configuration_(rect, config).retain()
         self.webview.pywebview_window = window
 
         self._browserDelegate = BrowserView.BrowserDelegate.alloc().init().retain()
@@ -539,10 +609,10 @@ class BrowserView:
         self.webview.setNavigationDelegate_(self._browserDelegate)
         self.window.setDelegate_(self._windowDelegate)
 
-        config = self.webview.configuration()
         config.userContentController().addScriptMessageHandler_name_(
             self._browserDelegate, 'browserDelegate'
         )
+
         self.datastore = WebKit.WKWebsiteDataStore.defaultDataStore()
 
         if _state['private_mode']:
@@ -568,7 +638,7 @@ class BrowserView:
 
         config.preferences().setValue_forKey_(webview_settings['ALLOW_FILE_URLS'], 'allowFileAccessFromFileURLs')
 
-        if _state['debug'] and webview_settings['OPEN_DEVTOOLS_IN_DEBUG']:
+        if _state['debug']:
             config.preferences().setValue_forKey_(True, 'developerExtrasEnabled')
 
         self.js_bridge = BrowserView.JSBridge.alloc().initWithObject_(window)
@@ -652,11 +722,8 @@ class BrowserView:
         self.shown.set()
 
         if not BrowserView.app.isRunning():
-            # Reset the application menu to the defaults
-            self._clear_main_menu()
-            self._add_app_menu()
-            self._add_view_menu()
-            self._add_user_menu()
+            new_menu = self._recreate_menus(self.menu)
+            BrowserView.app.setMainMenu_(new_menu)
 
             BrowserView.app.activateIgnoringOtherApps_(Foundation.YES)
             AppHelper.installMachInterrupt()
@@ -670,19 +737,13 @@ class BrowserView:
         AppHelper.callAfter(_show)
 
     def hide(self):
-        def _hide():
-            self.window.orderOut_(self.window)
-
-        AppHelper.callAfter(_hide)
+        AppHelper.callAfter(self.window.orderOut_, self.window)
 
     def destroy(self):
         AppHelper.callAfter(self.window.close)
 
     def set_title(self, title):
-        def _set_title():
-            self.window.setTitle_(title)
-
-        AppHelper.callAfter(_set_title)
+        AppHelper.callAfter(self.window.setTitle_, title)
 
     def toggle_fullscreen(self):
         def toggle():
@@ -726,10 +787,10 @@ class BrowserView:
         self.resize(width, height, None)
 
     def minimize(self):
-        self.window.miniaturize_(self)
+        AppHelper.callAfter(self.window.miniaturize_, self)
 
     def restore(self):
-        self.window.deminiaturize_(self)
+        AppHelper.callAfter(self.window.deminiaturize_, self)
 
     def move(self, x, y):
         flipped_y = self.screen.size.height - y
@@ -839,7 +900,7 @@ class BrowserView:
         def create_dialog(*args):
             dialog_type = args[0]
 
-            if dialog_type == SAVE_DIALOG:
+            if dialog_type == FileDialog.SAVE:
                 save_filename = args[2]
 
                 save_dlg = AppKit.NSSavePanel.savePanel()
@@ -861,26 +922,34 @@ class BrowserView:
                 open_dlg = AppKit.NSOpenPanel.openPanel()
 
                 # Enable the selection of files in the dialog.
-                open_dlg.setCanChooseFiles_(dialog_type != FOLDER_DIALOG)
+                open_dlg.setCanChooseFiles_(dialog_type != FileDialog.FOLDER)
 
                 # Enable the selection of directories in the dialog.
-                open_dlg.setCanChooseDirectories_(dialog_type == FOLDER_DIALOG)
+                open_dlg.setCanChooseDirectories_(dialog_type == FileDialog.FOLDER)
 
                 # Enable / disable multiple selection
                 open_dlg.setAllowsMultipleSelection_(allow_multiple)
 
                 # Set allowed file extensions
                 if file_filter:
-                    open_dlg.setAllowedFileTypes_(file_filter[0][1])
+                    if isinstance(file_filter, Foundation.WKNSArray):
+                        try:
+                            import UniformTypeIdentifiers
+                            UTType = UniformTypeIdentifiers.UTType
+                        except ImportError:
+                            UTType = None # Fallback if UTType is not available
 
-                    # Add a menu to choose between multiple file filters
-                    if len(file_filter) > 1:
-                        filter_chooser = BrowserView.FileFilterChooser.alloc().initWithFilter_(
-                            file_filter
-                        )
-                        filter_chooser.setFileDialog_(open_dlg)
-                        open_dlg.setAccessoryView_(filter_chooser)
-                        open_dlg.setAccessoryViewDisclosed_(True)
+                        open_dlg.setAllowedContentTypes_([UTType.typeWithMIMEType_("image/jpg")])
+                    else:
+                        open_dlg.setAllowedFileTypes_(file_filter[0][1])
+
+                        if len(file_filter) > 1:
+                            filter_chooser = BrowserView.FileFilterChooser.alloc().initWithFilter_(
+                                file_filter
+                            )
+                            filter_chooser.setFileDialog_(open_dlg)
+                            open_dlg.setAccessoryView_(filter_chooser)
+                            open_dlg.setAccessoryViewDisclosed_(True)
 
                 if directory:  # set initial directory
                     open_dlg.setDirectoryURL_(Foundation.NSURL.fileURLWithPath_(directory))
@@ -902,98 +971,37 @@ class BrowserView:
 
         return self._file_name
 
+    def _recreate_menus(self, user_menu):
+        main_menu = self._clear_main_menu()
+        self._add_app_menu(main_menu)
 
-    def _add_user_menu(self):
-        """
-        Create a custom menu for the app menu (MacOS bar menu)
+        if webview_settings['SHOW_DEFAULT_MENUS']:
+            self._add_view_menu(main_menu)
+            self._add_edit_menu(main_menu)
 
-        Args:
-            app_menu_list ([webview.menu.Menu])
-        """
+        self._add_custom_menu(main_menu, user_menu)
 
-        if BrowserView.app_menu_list is None:
-            return
-
-        # From https://github.com/r0x0r/pywebview/issues/500
-        class InternalMenu:
-            def __init__(self, title, parent):
-                self.m = AppKit.NSMenu.alloc().init()
-                self.item = AppKit.NSMenuItem.alloc().init()
-                self.item.setSubmenu_(self.m)
-                if not isinstance(parent, self.__class__):
-                    self.m.setTitle_(title)
-                    parent.addItem_(self.item)
-                else:
-                    self.item.setTitle_(title)
-                    parent.m.addItem_(self.item)
-
-            def action(self, title: str, action: Callable, command: str | None = None):
-                InternalAction(self, title, action, command)
-                return self
-
-            def separator(self):
-                self.m.addItem_(AppKit.NSMenuItem.separatorItem())
-                return self
-
-            def sub_menu(self, title: str):
-                return self.__class__(title, parent=self)
-
-        class InternalAction:
-            def __init__(self, parent: InternalMenu, title: str, action: callable, command=None):
-                self.action = action
-                s = selector(self._call_action, signature=b'v@:')
-                if command:
-                    item = parent.m.addItemWithTitle_action_keyEquivalent_(title, s, command)
-                else:
-                    item = AppKit.NSMenuItem.alloc().init()
-                    item.setAction_(s)
-                    item.setTitle_(title)
-                    parent.m.addItem_(item)
-                item.setTarget_(self)
-
-            def _call_action(self):
-                # Don't run action function on main thread
-                Thread(target=self.action).start()
-
-        def create_submenu(title, line_items, supermenu):
-            m = InternalMenu(title, parent=supermenu)
-            for menu_line_item in line_items:
-                if isinstance(menu_line_item, MenuSeparator):
-                    m = m.separator()
-                elif isinstance(menu_line_item, MenuAction):
-                    m = m.action(menu_line_item.title, menu_line_item.function)
-                elif isinstance(menu_line_item, Menu):
-                    create_submenu(menu_line_item.title, menu_line_item.items, m)
-
-        os_bar_menu = BrowserView.app.mainMenu()
-        if os_bar_menu is None:
-            os_bar_menu = AppKit.NSMenu.alloc().init()
-            BrowserView.app.setMainMenu_(os_bar_menu)
-
-        for app_menu in BrowserView.app_menu_list:
-            create_submenu(app_menu.title, app_menu.items, os_bar_menu)
+        return main_menu
 
     def _clear_main_menu(self):
         """
-        Remove all items from the main menu.
+        Remove all items from the main menu or create a new one if it doesn't exist.
         """
         mainMenu = BrowserView.app.mainMenu()
         if mainMenu:
             mainMenu.removeAllItems()
 
-    def _add_app_menu(self):
+        mainMenu = AppKit.NSMenu.alloc().init()
+
+        return mainMenu
+
+    def _add_app_menu(self, mainMenu):
         """
         Create a default Cocoa menu that shows 'Services', 'Hide',
         'Hide Others', 'Show All', and 'Quit'. Will append the application name
         to some menu items if it's available.
         """
 
-        mainMenu = BrowserView.app.mainMenu()
-
-        if not mainMenu:
-            mainMenu = AppKit.NSMenu.alloc().init()
-            BrowserView.app.setMainMenu_(mainMenu)
-
         # Create an application menu and make it a submenu of the main menu
         mainAppMenuItem = AppKit.NSMenuItem.alloc().init()
         # The application menu is the first item, so add this menu ast the first item
@@ -1038,15 +1046,11 @@ class BrowserView:
             self._append_app_name(self.localization['cocoa.menu.quit']), 'terminate:', 'q'
         )
 
-    def _add_view_menu(self):
+    def _add_view_menu(self, mainMenu):
+
         """
         Create a default View menu that shows 'Enter Full Screen'.
         """
-        mainMenu = BrowserView.app.mainMenu()
-
-        if not mainMenu:
-            mainMenu = AppKit.NSMenu.alloc().init()
-            BrowserView.app.setMainMenu_(mainMenu)
 
         # Create an View menu and make it a submenu of the main menu
         viewMenu = AppKit.NSMenu.alloc().init()
@@ -1064,6 +1068,72 @@ class BrowserView:
             AppKit.NSControlKeyMask | AppKit.NSCommandKeyMask
         )
 
+    def _add_edit_menu(self, mainMenu):
+        """
+        Create a default Edit menu that shows Copy/Paste/etc.
+        """
+        # Create an Edit menu and make it a submenu of the main menu
+        editMenu = AppKit.NSMenu.alloc().init()
+        editMenu.setTitle_(self.localization['cocoa.menu.edit'])
+        editMenuItem = AppKit.NSMenuItem.alloc().init()
+        editMenuItem.setSubmenu_(editMenu)
+        # Make the edit menu the first item after the application menu
+        mainMenu.insertItem_atIndex_(editMenuItem, 1)
+
+        for (title, action, keyEquivalent) in [
+            (self.localization['cocoa.menu.cut'], 'cut:', 'x'),
+            (self.localization['cocoa.menu.copy'], 'copy:', 'c'),
+            (self.localization['cocoa.menu.paste'], 'paste:', 'v'),
+            (self.localization['cocoa.menu.selectAll'], 'selectAll:', 'a'),
+        ]:
+            menuItem = editMenu.addItemWithTitle_action_keyEquivalent_(
+                title, action, keyEquivalent
+            )
+
+    def _add_custom_menu(self, mainMenu, app_menu_list):
+        """
+        Create a custom menu for the app menu (MacOS bar menu)
+        """
+
+        if app_menu_list is None:
+            return
+
+        def process_menu_items(menu_items, parent_menu):
+            for item in menu_items:
+                if isinstance(item, MenuSeparator):
+                    parent_menu.addItem_(AppKit.NSMenuItem.separatorItem())
+                elif isinstance(item, MenuAction):
+                    # Actions must be registered before application start. Otherwise they are disabled.
+                    # Menu handler is a workaround to register actions after application start
+                    random_id = str(uuid.uuid4())[:6]
+                    action_id = item.function.__name__ + '.' + random_id
+                    menu_handler.register_action(action_id, item.function)
+
+                    menu_item = AppKit.NSMenuItem.alloc().initWithTitle_action_keyEquivalent_(
+                        item.title, 'handleMenuAction:', ''
+                    )
+                    menu_item.setTarget_(menu_handler)
+                    menu_item.setRepresentedObject_(action_id)
+                    parent_menu.addItem_(menu_item)
+                elif isinstance(item, Menu):
+                    submenu = AppKit.NSMenu.alloc().init()
+                    submenu.setTitle_(item.title)
+                    menu_item = AppKit.NSMenuItem.alloc().init()
+                    menu_item.setTitle_(item.title)
+                    menu_item.setSubmenu_(submenu)
+                    parent_menu.addItem_(menu_item)
+
+                    process_menu_items(item.items, submenu)
+
+        for app_menu in app_menu_list:
+            submenu = AppKit.NSMenu.alloc().init()
+            submenu.setTitle_(app_menu.title)
+            menu_item = AppKit.NSMenuItem.alloc().init()
+            menu_item.setTitle_(app_menu.title)
+            menu_item.setSubmenu_(submenu)
+            mainMenu.addItem_(menu_item)
+            process_menu_items(app_menu.items, submenu)
+
     def _append_app_name(self, val):
         """
         Append the application name to a string if it's available. If not, the
@@ -1128,6 +1198,29 @@ class BrowserView:
 
         return alert.runModal() == AppKit.NSAlertFirstButtonReturn
 
+    @staticmethod
+    def display_input_dialog(first_button, second_button, prompt, default_text):
+        AppKit.NSApplication.sharedApplication()
+        AppKit.NSRunningApplication.currentApplication().activateWithOptions_(
+            AppKit.NSApplicationActivateIgnoringOtherApps
+        )
+        alert = AppKit.NSAlert.alloc().init()
+        text_field = AppKit.NSTextField.alloc().initWithFrame_(
+            AppKit.NSMakeRect(0, 0, 240, 24)
+        )
+        text_field.cell().setScrollable_(True)
+        text_field.setStringValue_(default_text)
+        alert.setAccessoryView_(text_field)
+        alert.addButtonWithTitle_(first_button)
+        alert.addButtonWithTitle_(second_button)
+        alert.setMessageText_(prompt)
+        alert.setAlertStyle_(AppKit.NSWarningAlertStyle)
+
+        if alert.runModal() == AppKit.NSAlertFirstButtonReturn:
+            return text_field.stringValue()
+        else:
+            return None
+
     @staticmethod
     def should_close(window):
         quit = window.localization['global.quit']
@@ -1192,17 +1285,49 @@ class BrowserView:
             ctypes.create_string_buffer(signature_str), None, False
         )
 
+    @staticmethod
+    def _open_web_inspector(webview):
+        """
+        Programmatically open the Web Inspector for the given WKWebView.
+        Uses private WebKit APIs that may not work on all macOS versions.
+        """
+        try:
+            if hasattr(webview, '_inspector'):
+                inspector = webview._inspector()
+                if inspector and hasattr(inspector, 'show'):
+                    inspector.show()
+                    return True
+
+            return False
+
+        except Exception as e:
+            return False
+
     @staticmethod
     def quote(string):
         return string.replace(' ', '%20')
 
 
+class MenuHandler:
+    def __init__(self):
+        self.actions = {}
+
+    def handleMenuAction_(self, sender):
+        action_id = sender.representedObject()
+        if action_id in self.actions:
+            Thread(target=self.actions[action_id]).start()
+        else:
+            logger.warning(f'Menu cction {action_id} not found')
+
+    def register_action(self, action_id, action_callable):
+        self.actions[action_id] = action_callable
+
+menu_handler = MenuHandler()
+
+
 def setup_app():
-    # MUST be called before create_window and set_app_menu
     pass
 
-def set_app_menu(app_menu_list):
-    BrowserView.app_menu_list = app_menu_list
 
 def get_active_window():
     active_window = BrowserView.app.keyWindow()
